/**
    @file ft51_dma.h

    @brief Library header for DMA controller API.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_DMA_H__
#define __FT51_DMA_H__

#include <stdint.h>
#include "ft51.h"
#include "registers/ft51_dma_registers.h"

/** Convenience value for variables representing an unallocated DMA engine.**/
#define DMA_ENGINE_NONE 255 

/**
    @enum  FT51_DMA_MODE
    @brief Modes of operation of the DMA engine.
**/
typedef enum
{
    /** Peripheral uses interrupt mode rather than the DMA engine. **/
    DMA_DISABLED, 
    /** Peripheral will acquire the appropriate DMAs and release them when 
        finished. **/
    DMA_ACQUIRE,
    /** Peripheral will acquire and retain exclusive use of DMA engines. **/
    DMA_RETAIN, 
} FT51_DMA_MODE;

/**
    @enum DMA_FIFO_SIZE
    @brief The buffer size when the DMA is used in FIFO mode.
**/
typedef enum 
{
    /** 64 byte buffer. **/
    DMA_FIFO_64, 
    /** 128 byte buffer. **/
    DMA_FIFO_128, 
    /** 256 byte buffer. **/
    DMA_FIFO_256, 
    /** 512 byte buffer. **/
    DMA_FIFO_512, 
} DMA_FIFO_SIZE;

/**
    @enum DMA_MODE
    @brief The DMA mode of operation.
**/
typedef enum
{
    /** Push a specified number of bytes to an FTDI peripheral **/
    DMA_MODE_PUSH = 0x00, 
    /** Pull a specified number of bytes from an FTDI peripheral **/
    DMA_MODE_PULL = 0x08, 
    /** Copy memory to memory **/
    DMA_MODE_MEMCPY = 0x10, 
    /** Continually pull data from an FTDI peripheral and store in RAM. **/
    DMA_MODE_FIFO = 0x18
} DMA_MODE;

/**
    @enum DMA_FCTRL
    @brief The peripheral to be flow-controlled.
**/
typedef enum
{
    DMA_FCTRL_UART = 0x00,
    DMA_FCTRL_SPIM = 0x10,
    DMA_FCTRL_SPIS = 0x20,
    DMA_FCTRL_PARALLEL = 0x30,
    DMA_FCTRL_DMA_FIFO = 0x40,
    DMA_FCTRL_NONE = 0x42
}
DMA_FCTRL;

/**
    @typedef DMA_complete_cb
    @brief   Type of function to be called when DMA transfer is complete.

    Specify a function of this type in DMA_config to be notified
    when a fixed-size DMA transfer is complete.

    @param user_data  Address of any data required by your handler.
**/
typedef void (*DMA_complete_cb)(void *user_data);

/**
    @typedef DMA_FIFO_full_cb
    @brief   Type of function to be called when DMA FIFO is full.

    Specify a function of this type in DMA_config to be notified
    when a FIFO-mode DMA engine has a full buffer.

    @param user_data  Address of any data required by your handler.
**/
typedef void (*DMA_FIFO_full_cb)(void *user_data);

/**
    @typedef DMA_FIFO_avail_cb
    @brief   Type of function to be called when DMA FIFO has new data.

    Specify a function of this type in DMA_config to be notified
    when a FIFO-mode DMA engine receives new data.

    @param user_data  Address of any data required by your handler.
**/
typedef void (*DMA_FIFO_avail_cb)(void *user_data);



/**
    @brief   Call once, to initialise the DMA library and reset it to 
             a known state.

    When finished, call DMA_finalise().  It is not valid to call
    any other DMA functions before this one.

    @return FT51_OK if successful.
**/                                  
FT51_STATUS DMA_initialise(void);

/**
    @brief      \par Acquire a DMA engine.

    Allocates one of the four DMA engines.
    Once allocated, configure the DMA engine with DMA_configure() 
    (or DMA_configure_FIFO()) before use.
    @param[out] engine_number Receives number of allocated engine.
    @return     \p FT51_OK on success, or FT51_UNAVAILABLE of all
                four are already allocated.
**/
FT51_STATUS DMA_acquire(uint8_t *engine_number);

/**
    @brief      \par Release a DMA engine.

    This function will release the DMA engine previously acquired 
                through \p DMA_acquire().
    @param[in]  engine_number The DMA engine to release.
**/
void DMA_release(uint8_t engine_number);

/**
    @brief      \par Configure a DMA engine.

    Configure the DMA engine previously acquired through 
    \p DMA_acquire().  A DMA engine must be configured before it 
    can be used for an operation.  Once an operation is complete 
    the DMA engine can be re-configured for another operation 
    with a subsequent call to this function.
    @param[in]  engine_number The DMA engine to configure.
    @param[in]  destination Address of buffer (or register) to receive data.
    @param[in]  source Address of buffer (or register) to supply data.
    @param[in]  transfer_size Number of bytes to transfer.
    @param[in]  mode Push, pull, or memcpy.  For FIFO, use DMA_configureFIFO().
    @param[in]  flow_control Register to manage rate of data flow.
**/
void  DMA_configure(uint8_t     engine_number,
                    uint16_t    destination,
                    uint16_t    source,
                    uint16_t    transfer_size,
                    DMA_MODE    mode,
                    DMA_FCTRL   flow_control);

/**
    @brief      \par Reset a DMA engine.

    Clears all the register associated with the specified 
    DMA engine.
    @param[in]  engine_number The DMA engine to reset.
    @return     Always \p FT51_OK.
**/
FT51_STATUS DMA_reset(uint8_t  engine_number);

/**
    @brief      \par Enable the DMA engine.

    Starts the DMA operation that was previously configured.
    @param[in]  engine_number The DMA engine to enable.
**/
void DMA_enable(uint8_t  engine_number);

/**
    @brief      \par Disable the DMA engine.

    Stops the on-going DMA operation.
    @param[in]  engine_number The DMA engine to enable.
**/
void DMA_disable(uint8_t  engine_number);

/**
    @brief      \par Enable the DMA engine.

    <tt>NOT FIFO MODE</tt>.  
    Blocks until the DMA operation that was previously enabled 
    completes.
    @param[in]  engine_number The DMA engine to wait for.
**/
void DMA_wait_on_complete(uint8_t  engine_number);

/**
    @brief      \par Purge FIFO DMA engine.

    <tt>FIFO MODE ONLY</tt>.  
    Clears any data that is currently in the FIFO buffer of the 
    current DMA.
    @param[in]  engine_number The FIFO DMA engine to clear.
    @return     \p FT51_OK on success, otherwise \p FT51_INVALID_PARAMETER.
**/
FT51_STATUS DMA_purge_fifo(uint8_t  engine_number);

/**
    @brief      \par Get the FIFO DMA data count.

    <tt>FIFO MODE ONLY</tt>.  
    Returns the number of bytes that are currently within the FIFO 
    DMA buffer.
    @param[in]  engine_number The FIFO DMA engine to check.
    @return     Number of bytes transferred.
**/
uint16_t DMA_get_fifo_count(uint8_t  engine_number);

/**
    @brief      \par Get the FIFO DMA data.

    <tt>FIFO MODE ONLY</tt>.  
    Copies data from the FIFO buffer to the user application.
    If there are less than \p length bytes available it will return
    however many bytes there are, check \p transferred for the 
    actual number.
    @param[in]  engine_number The FIFO DMA engine to read from.
    @param[out] data Storage allocated for \p length bytes to be read from FIFO
                 DMA.
    @param[in]  length The number of bytes to read.
    @param[out] transferred The actual number of bytes read.
    @return     \p FT51_OK on success, otherwise \p FT51_INVALID_PARAMETER.
**/
FT51_STATUS DMA_get_fifo_data(uint8_t  engine_number, 
                              uint8_t *data, 
                              const uint16_t length, 
                              uint16_t *transferred);

/**
    @brief      \par Configure a DMA engine to operate on a FIFO buffer.

    Configure the DMA engine previously acquired through 
    \p DMA_acquire().  A DMA engine must be configured before it 
    can be used for an operation.  Once an operation is complete 
    the DMA engine can be re-configured for another operation 
    with a subsequent call to this function.
    @param[in]  engine_number The DMA engine to configure.
    @param[in]  destination Address of buffer (or register) to receive data.
    @param[in]  source Address of buffer (or register) to supply data.
    @param[in]  transfer_size Number of bytes to transfer.
    @param[in]  flow_control Register to manage rate of data flow.
    @param[in]  afull_trigger Determines when to invoke the almost-full cb.
    @param[in]  FIFO_full_cb Invoked when FIFO is full.
    @param[in]  FIFO_avail_cb Invoked when FIFO has at least one byte.
    @param[in]  user_data Opaque user data to be supplied to the callbacks.    
**/
void  DMA_configure_fifo(uint8_t            engine_number,
                         uint16_t           destination,
                         uint16_t           source,
                         uint16_t           transfer_size,
                         uint8_t            flow_control,   
                         uint8_t            afull_trigger,
                         DMA_FIFO_full_cb   FIFO_full_cb,   
                         DMA_FIFO_avail_cb  FIFO_avail_cb,
                         void              *user_data);

/**
    @brief      \par Change the destination buffer for a FIFO-mode DMA engine.

    Reconfigure an existing DMA engine to use a different buffer.
    @param[in]  engine_number    The DMA engine to configure.
    @param[in]  new_destination  Address of buffer to receive data.
    @return     Final number of bytes transferred to previous destination.
**/
uint16_t  DMA_switch_fifo(uint8_t            engine_number,
                          void              *new_destination);

/**
    @brief      Check if transfer is complete

    Is transfer complete?
    @param[in]  engine_number The DMA engine to configure.
    @return     Non-zero if transfer is complete.
**/
uint8_t DMA_is_complete(uint8_t engine_number);

#endif // __FT51_DMA_H__