/**
    @file ft51_interrupts.h

    @brief    Interrupt handling library.

    Covers internal 8051 and external interrupts.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_INTERRUPTS_H__
#define __FT51_INTERRUPTS_H__

#include <stdint.h>
#include "ft51.h"

/**
    @typedef interrupts_handler
    @brief   Type of function to be called when execution is interrupted.

    Supply a function of this type to interrupts_register().  
    Your function will be invoked when the relevant peripheral 
    interrupts.

    @param[in] info  If applicable, indicates which sub-peripheral interrupted.
**/
typedef void (*interrupts_handler)(const uint8_t info);

/**
    @enum interrupts_peripheral
    @brief The peripheral whose interrupts are to be recognised.
**/
typedef enum
{
    interrupts_DMA,
    interrupts_timer0,
    interrupts_timer1,
    interrupts_timer2,
    interrupts_serial,
    interrupts_FT122,
    interrupts_IO_cell, 
    interrupts_UART,
    interrupts_SPI_master,
    interrupts_SPI_slave, 
    interrupts_PWM, 
    interrupts_timer, 
    interrupts_parallel245,
    interrupts_I2C_master,
    interrupts_I2C_slave
}
interrupts_peripheral;

/** Assign to FT51_INTERRUPTS to include DMA interrupt-processing code. **/
#define FT51_INT_DMA           1
/** Assign to FT51_INTERRUPTS to include Timer0 interrupt-processing code. **/
#define FT51_INT_TIMER0        2
/** Assign to FT51_INTERRUPTS to include Timer1 interrupt-processing code. **/
#define FT51_INT_TIMER1        4
/** Assign to FT51_INTERRUPTS to include code for processing interrupts from 
    AIO cells, UART, SPI, PWM, TimerA/B/C/D and Parallel245. **/
#define FT51_INT_PERIPHERALS   8
/** Assign to FT51_INTERRUPTS to include I2C Master interrupt-processing code. **/
#define FT51_INT_I2CM         16
/** Assign to FT51_INTERRUPTS to include I2C Slave interrupt-processing code. **/
#define FT51_INT_I2CS         32
/** Assign to FT51_INTERRUPTS to include Timer2 interrupt-processing code. **/
#define FT51_INT_TIMER2       64
/** Assign to FT51_INTERRUPTS to include Serial interrupt-processing code. **/
#define FT51_INT_SERIAL      128 

/** To avoid bloating the binary, selectively include interrupt-processing by 
    building with FT51_INTERRUPTS set to a non-zero value.  For example, build 
    with FT_INTERRUPTS=10 to enable Timer0 and the FTDI peripherals (SPI, 
    USB etc.).  By default, all interrupt-processing code is included.
**/
#ifndef FT51_INTERRUPTS
#define FT51_INTERRUPTS (FT51_INT_DMA + FT51_INT_TIMER0 + FT51_INT_TIMER1 + \
                         FT51_INT_PERIPHERALS + FT51_INT_I2CM + FT51_INT_I2CS + \
                         FT51_INT_TIMER2 + FT51_INT_SERIAL)
#endif // FT51_INTERRUPTS

/* Section 3.10 of the SDCC manual states that ISRs may be in any source file
 * if their prototypes are included by the file containing main().
 */
/**
    Interrupt Service Routine for DMA engines 0, 1, 2 and 3.
**/
#if (FT51_INTERRUPTS & FT51_INT_DMA) == FT51_INT_DMA
void INT0_ISR(void) __interrupt (0);
#endif // (FT51_INTERRUPTS & FT51_INT_DMA) == FT51_INT_DMA

/**
    Interrupt Service Routine for standard 8051 Timer0.
**/
#if (FT51_INTERRUPTS & FT51_INT_TIMER0) == FT51_INT_TIMER0
void Timer0_ISR(void) __interrupt (1);
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER0) == FT51_INT_TIMER0

/**
    Interrupt Service Routine for AIO cells, UART, SPI, PWM, 
    TimerA/B/C/D and Parallel245. 
**/
#if (FT51_INTERRUPTS & FT51_INT_PERIPHERALS) == FT51_INT_PERIPHERALS
void INT1_ISR(void) __interrupt (2);
#endif // (FT51_INTERRUPTS & FT51_INT_PERIPHERALS) == FT51_INT_PERIPHERALS

/**
    Interrupt Service Routine for standard 8051 Timer1.
**/
#if (FT51_INTERRUPTS & FT51_INT_TIMER1) == FT51_INT_TIMER1
void Timer1_ISR(void) __interrupt (3);
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER1) == FT51_INT_TIMER1

/**
    Interrupt Service Routine for standard 8051 serial port.
**/
#if (FT51_INTERRUPTS & FT51_INT_SERIAL) == FT51_INT_SERIAL
void Serial_ISR(void) __interrupt (4);
#endif // (FT51_INTERRUPTS & FT51_INT_SERIAL) == FT51_INT_SERIAL

/**
    Interrupt Service Routine for third 8051 Timer.
**/
#if (FT51_INTERRUPTS & FT51_INT_TIMER2) == FT51_INT_TIMER2
void Timer2_ISR(void) __interrupt (5);
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER2) == FT51_INT_TIMER2

/**
    Interrupt Service Routine for I2C Master.
**/
#if (FT51_INTERRUPTS & FT51_INT_I2CM) == FT51_INT_I2CM
void I2C_MASTER_ISR(void) __interrupt (13);
#endif // (FT51_INTERRUPTS & FT51_INT_I2CM) == FT51_INT_I2CM

/**
    Interrupt Service Routine for I2C Slave.
**/
#if (FT51_INTERRUPTS & FT51_INT_I2CS) == FT51_INT_I2CS
void I2C_SLAVE_ISR(void) __interrupt (14);
#endif // (FT51_INTERRUPTS & FT51_INT_I2CS) == FT51_INT_I2CS


/**
    Register a handler to be invoked when the specified peripheral 
    interrupts execution.
    
    @param[in] isr         The routine to service the interrupt.
    @param[in] peripheral  Specifies the peripheral.
**/
void interrupts_register(interrupts_handler    isr, 
                         interrupts_peripheral peripheral);

#endif // __FT51_INTERRUPTS_H__